// this file contains: interpreter pluggable primitive support functions
//
// author: Colin Sarsfield
//
// Please read the file named License for license information.
// 

#include <image.h>
#include <Path.h>

extern "C" {
	#include "sq.h"
}

#include "sqBeApp.h"
#include "sqBeExternalPrims.h"

/* The next three functions must be implemented by sqXYZExternalPrims.c */
/* ioLoadModule:
	Load a module from disk.
	WARNING: this always loads a *new* module. Don't even attempt to find a loaded one.
	WARNING: never primitiveFail() within, just return 0
*/
int ioLoadModule(char *pluginName) 
{
	char modulePath[B_PATH_NAME_LENGTH];
	image_id id;
	strcpy(modulePath, VMPath);
	strcat(modulePath, pluginName);
	id = load_add_on(modulePath);
	if (id <= 0) {
		//printf("couldn't load module %s\n", pluginName);
		return 0;
	} else {
		//printf("loaded module %s (%d)\n", pluginName, id);
		return id;
	}
}

/* ioFindExternalFunctionIn:
	Find the function with the given name in the moduleHandle.
	WARNING: never primitiveFail() within, just return 0.
*/
int ioFindExternalFunctionIn(char *lookupName, int moduleHandle) 
{
	void *functionPtr;
	if (get_image_symbol((image_id)moduleHandle, lookupName, 
			B_SYMBOL_TYPE_TEXT, &functionPtr) == B_OK) {
		//printf("Found symbol %s\n", lookupName);
		return (int) functionPtr;
	} else {
		//printf("Didn't find symbol %s\n", lookupName);
		return 0;
	}
}

/* ioFreeModule:
	Free the module with the associated handle.
	WARNING: never primitiveFail() within, just return 0.
*/
int ioFreeModule(int moduleHandle) 
{
	unload_add_on(moduleHandle);
	//printf("unloaded module %d\n", moduleHandle);
	return 1;
}
