#if !defined(SQBEGLOBAL_H)
#define SQBEGLOBAL_H

#include <OS.h>

extern port_id SqSem;
extern port_id SqEvents;
extern int SqEventSem;
extern volatile bool Quitting;

// comment this next line out if you do not want stdout debuging messages turned on
//#define DEBUG_SQUEAK

// Profiling stuff
// comment out "#define PROFILE" to disable

//#define PROFILE
#define MAX_PROFILE_POINTS 10
#define PROFILE_INIT_M1    {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
#define PROFILE_NAMES      {"Draw to window", "CopyRect", "sqAccept", "accept", "snooze", "misc", "DeferedRegion.Include", "Invalidate", "Defered update", "Immedidate update"}

#if defined(PROFILE)
# define PROF_ENT(x, y)		ProfileEntry(x, y);
#else
# define PROF_ENT(x, y)
#endif

void ProfileEntry(int id, bigtime_t time);

// standard stubs for unimplemented Smalltalk primitives

#define STUBBED_OUT		{ DEBUG_OUT("STUBBED_OUT"); success(false); DEBUG_OUT("STUBBED_OUT/ret"); }
#define STUBBED_OUT_INT { success(false); return 0; }
#define STUBBED_OUT_DBL { success(false); return 0.0; }

extern "C" { 
	#include "sq.h"
	int printCallStack(void);     // this prints out the current call stack in Squeak
	                              // useful for debugging infinite loops (hangs)
	                              // resulting from primitive failure
	int displayObject(void);
	int isPointers(int oop);
	int fetchPointerofObject(int fieldIndex, int oop);
	int lengthOf(int oop);
}

#if defined(DEBUG_SQUEAK)
# define DEBUG_OUT(x)	printf("%s\n", x)
#else
# define DEBUG_OUT(x)
#endif

// Variables -- Imported from Virtual Machine
extern "C" {  
	extern int fullScreenFlag;          // full screen mode y/n
	extern int interruptCheckCounter;
	extern int interruptKeycode;
	extern int interruptPending;        // set to true by recordKeystroke if interrupt key is pressed
	extern int savedWindowSize;         // set from header when image file is loaded
	extern int deferDisplayUpdates;     // interpreter is defering display updates, (Morphic only)
	}
	
typedef enum { log_info, log_warning, log_error} log_msg;

void log_message(log_msg msg_type, char *msg);

#endif
