// this file contains: Miscellaneous interpreter support functions
//
// author: Colin Sarsfield
//
// Please read the file named License for license information.
// 

#include "sqBeApp.h"
#include "sqBeGlobal.h"

#include <Clipboard.h>

const char *GetAttributeString(int id);
char *GetClipboardString();

/*** tablet support ***/
int tabletGetParameters(int cursorIndex, int result[]) {
	success(false);
	return 0;
}
	
int tabletRead(int cursorIndex, int result[]) {
	success(false);
	return 0;
}

int tabletResultSize(void) {
	success(false);
	return 0;
}


///////////////////////////////////
// interpreter defined functions //
///////////////////////////////////
		
int attributeSize(int id) {
	DEBUG_OUT("attributeSize");
	DEBUG_OUT("attributeSize/ret");
	return strlen(GetAttributeString(id));
}

int clipboardReadIntoAt(int count, int byteArrayIndex, int startIndex) {
	DEBUG_OUT("clipboardReadIntoAt");
	char *clipStr = GetClipboardString();
	memcpy((char *)byteArrayIndex + startIndex, clipStr, count);
	free(clipStr);
	DEBUG_OUT("clipboardReadIntoAt/ret");
	return 0;
}

int clipboardSize(void) {
	DEBUG_OUT("clipboardSize");
	char *clipStr = GetClipboardString();
	int i = strlen(clipStr);
	free(clipStr);
	DEBUG_OUT("clipboardSize/ret");
	return i;
}

int clipboardWriteFromAt(int count, int byteArrayIndex, int startIndex) {
	DEBUG_OUT("clipboardWriteFromAt");
	if ( be_clipboard->Lock() ) 
		{	 
		be_clipboard->Clear(); 
		BMessage *clipper = be_clipboard->Data(); 

		unsigned char *dst, *src, *dsto;

		src = (unsigned char *)byteArrayIndex + startIndex;
		dsto = dst = (unsigned char *)malloc(count + 1);
		src = (unsigned char *)byteArrayIndex + startIndex;

		while(count--)
			{
			// remap cr's to lf's
			if (*src == 13)
				{
				src++;
				*dst++ = 10;
				}
			else
				*dst++ = *src++;
			}

		*dst = '\0';
  
		clipper->AddString("text", (char *)dsto); 
		clipper->AddData("text/plain", B_MIME_TYPE, dsto, strlen((char *)dsto));
		clipper->what = B_SIMPLE_DATA;
		
		free(dsto);
		
		be_clipboard->Commit(); 
		be_clipboard->Unlock(); 
		
		}	
	DEBUG_OUT("clipboardWriteFromAt/ret");
	return 1;
}

int getAttributeIntoLength(int id, int byteArrayIndex, int length) {
	const char *srcPtr, *end;
	char *dstPtr;
	int charsToMove;

	DEBUG_OUT("getAttributeIntoLength");
	srcPtr = GetAttributeString(id);
	charsToMove = strlen(srcPtr);
	if (charsToMove > length) {
		charsToMove = length;
	}

	dstPtr = (char *) byteArrayIndex;
	end = srcPtr + charsToMove;
	while (srcPtr < end) {
		*dstPtr++ = *srcPtr++;
	}
	DEBUG_OUT("getAttributeIntoLength/ret");
	return charsToMove;
}

int imageNameGetLength(int sqImageNameIndex, int length) {
	char *sqImageName = (char *) sqImageNameIndex;
	int count, i;
	DEBUG_OUT("imageNameGetLength");

	count = strlen(imageName);
	count = (length < count) ? length : count;

	/* copy the file name into the Squeak string */
	for (i = 0; i < count; i++) {
		sqImageName[i] = imageName[i];
	}
	DEBUG_OUT("imageNameGetLength/ret");
	return count;
}

int imageNamePutLength(int sqImageNameIndex, int length) {
	char *sqImageName = (char *) sqImageNameIndex;
	int count, i, ch, j;
	int lastColonIndex = -1;

	DEBUG_OUT("imageNamePutLength");
	count = (B_FILE_NAME_LENGTH < length) ? B_FILE_NAME_LENGTH : length;

	/* copy the file name into a null-terminated C string */
	for (i = 0; i < count; i++) {
		ch = imageName[i] = sqImageName[i];
		if (ch == ':') {
			lastColonIndex = i;
		}
	}
	imageName[count] = 0;

	/* copy short image name into a null-terminated C string */
	for (i = lastColonIndex + 1, j = 0; i < count; i++, j++) {
		ShortImageName[j] = imageName[i];
	}
	ShortImageName[j] = 0;

	SqWindow->SetTitle(ShortImageName);

	DEBUG_OUT("imageNamePutLength/ret");
	return count;
}

int imageNameSize(void) {
	DEBUG_OUT("imageNameSize");
	DEBUG_OUT("imageNameSize/ret");
	return strlen(imageName);
}

int vmPathGetLength(int sqVMPathIndex, int length) {
	char *stVMPath = (char *) sqVMPathIndex;
	int count, i;

	DEBUG_OUT("vmPathGetLength");
	count = strlen(VMPath);
	count = (length < count) ? length : count;

	/* copy the file name into the Squeak string */
	for (i = 0; i < count; i++) {
		stVMPath[i] = VMPath[i];
	}
	DEBUG_OUT("vmPathGetLength/ret");
	return count;
}

int vmPathSize(void)
	{
	DEBUG_OUT("vmPathSize");
	DEBUG_OUT("vmPathSize/ret");
	return strlen(VMPath);
	}

/////////////////////
// Profiling Stubs //
/////////////////////

int clearProfile(void)														STUBBED_OUT_INT
int dumpProfile(void)														STUBBED_OUT_INT
int startProfiling(void)													STUBBED_OUT_INT
int stopProfiling(void)														STUBBED_OUT_INT

// support functions

const char * GetAttributeString(int id) {
	DEBUG_OUT("GetAttributeString");
	const char * c = "";
	/* This is a hook for getting various status strings back from
	   the OS. In particular, it allows Squeak to be passed arguments
	   such as the name of a file to be processed. Command line options
	   are reported this way as well, on platforms that support them.
	*/
	// the following comment is copied directly from the Squeak source
	/* Optional. Answer the string for the system attribute with the given 
	integer ID. Answer nil if the given attribute is not defined on this 
	platform. On platforms that support invoking programs from command 
	lines (e.g., Unix), this mechanism can be used to pass command line 
	arguments to programs written in Squeak. By convention, the first 
	command line argument that is not a VM configuration option is 
	considered a 'document' to be filed in. Such a document can add 
	methods and classes, can contain a serialized object, can include code to 
	be executed, or any combination of these. 
	Currently defined attributes include: 
	-1000...-1 - command line arguments that specify VM options 
	0 - the full path name for currently executing VM 
	(or, on some platforms, just the path name of the VM's directory) 
	1 - full path name of this image 
	2 - a Squeak document to open, if any 
	3...1000 - command line arguments for Squeak programs 
	1001 - this platform's operating system 
	1002 - operating system version 
	1003 - this platform's processor type */
	switch (id)
		{
		case 0:	// id #0 should return the full name of VM
			c = ProgName;
			break;
		case 1:	// id #1 should return imageName
			c = imageName;
			break;
		case 2: // id #2 defines file to load automatically
			c = "";
			break;
		case 1001: // id #1001 should return OS name
			c = "BeOS";
			break;
		case 1002: // id #1002 should return OS version
			c = "";
			break;
		case 1003: // id #1003 should return processor type
			system_info info;
			get_system_info(&info);
			switch (info.cpu_type)
				{
				case B_CPU_PPC_601: c = "PowerPC 601"; break;
				case B_CPU_PPC_603: c = "PowerPC 603"; break;
				case B_CPU_PPC_603e: c = "PowerPC 603e"; break;
				case B_CPU_PPC_604: c = "PowerPC 604"; break;
				case B_CPU_PPC_604e: c = "PowerPC 604e"; break;
				case B_CPU_PPC_686: c = "PowerPC 686"; break;
				case B_CPU_AMD_29K: c = "AMD 29K"; break;
				case B_CPU_X86: c = "x86"; break;
				case B_CPU_MC6502: c = "MC6502"; break;
				case B_CPU_Z80: c = "Z80"; break;
				case B_CPU_ALPHA: c = "Alpha"; break;
				case B_CPU_MIPS: c = "Mips"; break;
				case B_CPU_HPPA: c = "HPPA"; break;
				case B_CPU_M68K: c = "M68K"; break;				
				case B_CPU_ARM: c = "ARM"; break;
				case B_CPU_SH: c = "SH"; break;
				case B_CPU_SPARC: c = "SPARC"; break;
				case B_CPU_INTEL_X86: c = "Intel x86"; break;
				case B_CPU_INTEL_PENTIUM: c = "Intel Pentium"; break;
				case B_CPU_INTEL_PENTIUM75: c = "Intel Pentium 75"; break;
				case B_CPU_INTEL_PENTIUM_486_OVERDRIVE: c = "Intel Pentium 486 Overdrive"; break;
				case B_CPU_INTEL_PENTIUM_MMX: c = "Intel Pentium MMX"; break;
				case B_CPU_INTEL_PENTIUM75_486_OVERDRIVE: c = "Intel Pentium 75 486 Overdrive"; break;
				case B_CPU_INTEL_PENTIUM_PRO: c = "Intel Pentium Pro"; break;
				case B_CPU_INTEL_PENTIUM_II_MODEL_3: c = "Intel Pentium II m3"; break;
				case B_CPU_INTEL_PENTIUM_II_MODEL_5: c = "Intel Pentium II m5"; break;
				case B_CPU_AMD_X86: c = "AMD x86"; break;
				case B_CPU_AMD_K5_MODEL0: c = "AMD K5 m0"; break;
				case B_CPU_AMD_K5_MODEL1: c = "AMD K5 m1"; break;
				case B_CPU_AMD_K5_MODEL2: c = "AMD K5 m2"; break;
				case B_CPU_AMD_K5_MODEL3: c = "AMD K5 m3"; break;
				case B_CPU_AMD_K6_MODEL6: c = "AMD K6 m6"; break;
				case B_CPU_AMD_K6_MODEL7: c = "AMD K6 m7"; break;
				case B_CPU_AMD_K6_MODEL8: c = "AMD K6 m8"; break;
				case B_CPU_AMD_K6_MODEL9: c = "AMD K6 m9"; break;
				case B_CPU_CYRIX_X86: c = "Cyrix x86"; break;
				case B_CPU_CYRIX_GXm: c = "Cyrix GXm"; break;
				case B_CPU_CYRIX_6x86MX: c = "Cyrix 6x86MX"; break;
				default: c = "Unknown";
				} 			
			break;
		default:
			// VM args
			if (id > -PubImageArg && id <= -1)
				c = (*Pubargv)[PubImageArg + id];
			// image args
			else if (id >= 3 && id <= (*Pubargc - PubImageArg + 1))
				c = (*Pubargv)[PubImageArg + id - 2];
			else
				// attribute undefined by this platform
				success(false);
		}

	DEBUG_OUT("GetAttributeString/ret");
	return c;
}

char *GetClipboardString()
	{
	char *theText;
	
	if ( be_clipboard->Lock() ) 
		{ 
		BMessage *clipper = be_clipboard->Data(); 
		ssize_t numBytes;
		char *data, *wTheText;
		bool hasData;
		
		hasData = clipper->FindData("text/plain", B_MIME_TYPE, (const void **)&data, &numBytes) == B_OK;
		if (!hasData)
			hasData = clipper->FindData("text", B_STRING_TYPE, (const void **)&data, &numBytes) == B_OK;
		if (hasData)
			{
			theText = wTheText = (char *)malloc(numBytes + 1);
			while(numBytes--)
				{
				// remap lf's to cr's
				if (*data == 10)
					{
					data++;
					*wTheText++ = 13;
					}
				else
					*wTheText++ = *data++;
				}

			*wTheText = '\0';
			}
		else
			{
			theText = (char *)malloc(1);
			*theText = '\0';
			}

		be_clipboard->Unlock(); 
		}
	else
		{
		theText = (char *)malloc(1);
		*theText = '\0';
		}
		
	return theText;
	}

