// this file contains: BWindow support
//
// authors: Trey Bordeau, Colin Sarsfield
//
// Please read the file named License for license information.
// 

#include "sqBeApp.h"
#include "sqBeWindow.h"
#include "sqBeView.h"
#include "sqBeSharedWindow.h"
#include "sqBeGlobal.h"

#include <MessageQueue.h>

//////////////////
// SqueakWindow //
//////////////////

SqueakWindow::SqueakWindow(BRect frame) 
		: BWindow(frame, "BeSqueak", B_TITLED_WINDOW, B_OUTLINE_RESIZE)
	{
	DEBUG_OUT("SqueakWindow");
	
	// add the views
	BRect bounds = Bounds();
	CreateViews(this, bounds);
	
	DEBUG_OUT("SqueakWindow/ret");
	}

void SqueakWindow::DispatchMessage(BMessage *msg, BHandler *target) 
	{ 
	if (HandleDispatchMessage(this, msg, target))
		BWindow::DispatchMessage(msg, target);
	}		
		
void SqueakWindow::FrameResized(float width, float height)
	{
	HandleFrameResized(this, width, height);
	BWindow::FrameResized(width, height);
	}
	
void SqueakWindow::MessageReceived(BMessage *message) {
	DEBUG_OUT("MessageReceived");
	if (message->what == DO_INVALIDATE)
		{
		// gang up all of the DO_INVALIDATE messages, as our squeakview::Draw() knows
		// about clipping regions
		DisableUpdates();
		BRect drawRect;
		BMessage *lookahead;
		// handle the one we've been called for
		message->FindRect("drawRect", &drawRect);
		SqView->Invalidate(drawRect);
		// and any others that might be piled up in the queue
		while ((lookahead = MessageQueue()->FindMessage((uint32)DO_INVALIDATE)) != 0)
			{
			lookahead->FindRect("drawRect", &drawRect); 
			MessageQueue()->RemoveMessage(lookahead);
			SqView->Invalidate(drawRect);
			}
		// let 'er rip!
		EnableUpdates();
		}
	else
		BWindow::MessageReceived(message);
	DEBUG_OUT("MessageReceived/ret");
	}
		
bool SqueakWindow::QuitRequested()
	{
	HandleQuitRequested();
	return(Quitting);
	}
		
void SqueakWindow::WindowActivated(bool active)
	{
	DEBUG_OUT("WindowActivated");

	uint32 buttons;
	
	WindowActive = active;
	if (active)
		SqView->MakeFocus();
	else
		SqView->GetMouse(&SavedMousePosition, &buttons);		

	BWindow::WindowActivated(active);

	DEBUG_OUT("WindowActivated/ret");
	}
		
